@@ -36,11 +36,12 @@
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-meta.h>
 #include <libgnomeprint/gnome-print-transport.h>
-#include <libgnomeprint/gnome-print-multipage.h>
 #include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprint/gnome-print-config-private.h>
 #include <libgnomeprint/gnome-print-i18n.h>
 
+#include <libgnomeprint/gnome-print-filter.h>
+
 typedef struct _GnomePrintJobPrivate GnomePrintJobPrivate;
 
 struct _GnomePrintJob {
@@ -93,7 +94,7 @@ static void gnome_print_job_class_init (GnomePrintJobClass *klass);
 static void gnome_print_job_init (GnomePrintJob *job);
 static void gnome_print_job_finalize (GObject *object);
 
-static void job_update_layout_data (GnomePrintJob *job);
+static void gnome_print_job_update_layout_data (GnomePrintJob *job);
 static void job_parse_config_data (GnomePrintJob *job);
 static void job_clear_config_data (GnomePrintJob *job);
 
@@ -320,7 +321,69 @@ gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
 	g_return_if_fail (input_file != NULL);
 	g_return_if_fail (job->input_file == NULL);
 
-	job->input_file = g_strdup (input_file);
+	job->input_file = (guchar *) g_strdup (input_file);
+}
+
+static void
+gnome_print_job_setup_context (GnomePrintJob *job, GnomePrintContext *pc)
+{
+	GnomePrintFilter *filter = NULL;
+	gchar *description;
+
+	g_return_if_fail (GNOME_IS_PRINT_JOB (job));
+	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (pc));
+
+	g_object_set (G_OBJECT (pc), "filter", NULL, NULL);
+
+	/* Explicit filter description */
+	description = (gchar *) gnome_print_config_get (job->config,
+			(const guchar *) "Settings.Output.Job.Filter");
+	if (description) {
+		GError *e = NULL;
+
+		filter = gnome_print_filter_new_from_description (description, &e);
+		if (e) {
+			g_warning ("Problem while creating filter from '%s': %s",
+				description, e->message);
+			g_error_free (e);
+		}
+		g_free (description);
+		if (filter) {
+			g_object_set (G_OBJECT (pc), "filter", filter, NULL);
+			g_object_unref (G_OBJECT (filter));
+		}
+	}
+
+	gnome_print_job_update_layout_data (job);
+	if (job->priv->LY_LIST) {
+		GnomePrintFilter *multipage;
+		GValue vd = {0,};
+		GValueArray *va = g_value_array_new (0);
+		guint i, npages;
+		
+		npages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
+		npages = (npages + job->priv->num_affines - 1) / job->priv->num_affines;
+		g_value_init (&vd, G_TYPE_DOUBLE);
+		for (i = g_list_length (job->priv->LY_LIST); i > 0; i--) {
+			gdouble *a = g_list_nth_data (job->priv->LY_LIST, i - 1);
+			guint j;
+
+			for (j = 6; j > 0; j--) {
+				g_value_set_double (&vd, a[j - 1]);
+				g_value_array_prepend (va, &vd);
+			}
+		}
+		g_value_unset (&vd);
+		multipage = gnome_print_filter_new_from_module_name ("multipage",
+				"affines", va, NULL);
+		g_value_array_free (va);
+		if (multipage) {
+			if (filter)
+				gnome_print_filter_append_predecessor (filter, multipage);
+			g_object_set (G_OBJECT (pc), "filter", multipage, NULL);
+			g_object_unref (G_OBJECT (multipage));
+		}
+	}
 }
 
 /**
@@ -337,21 +400,24 @@ gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
 int
 gnome_print_job_get_pages (GnomePrintJob *job)
 {
-	gint mp;
+	gint n;
+	GnomePrintContext *pc;
 
-	g_return_val_if_fail (job != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), 0);
-	g_return_val_if_fail (job->meta, 0);
 	g_return_val_if_fail (GNOME_PRINT_JOB_CLOSED (job), 0);
 
-	job_update_layout_data (job);
+	/*
+	 * Filters can suppress or add pages. We therefore just pass everything
+	 * through the filter pipeline and count the number of pages that come 
+	 * out of it.
+	 */
+	pc = g_object_new (GNOME_TYPE_PRINT_META, NULL);
+	gnome_print_job_setup_context (job, pc);
+	gnome_print_meta_render (GNOME_PRINT_META (job->meta), pc);
+	n = gnome_print_meta_get_pages (GNOME_PRINT_META (pc));
+	g_object_unref (G_OBJECT (pc));
 
-	mp = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
-
-	if (job->priv->num_affines <= 1)
-		return mp;
-
-	return (mp + job->priv->num_affines - 1) / job->priv->num_affines;
+	return n;
 }
 
 /**
@@ -389,7 +455,7 @@ gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *heig
 	g_return_val_if_fail (width != NULL, FALSE);
 	g_return_val_if_fail (height != NULL, FALSE);
 
-	job_update_layout_data (job);
+	gnome_print_job_update_layout_data (job);
 
 	if (job->priv->LY_LIST) {
 		if (width)
@@ -450,7 +516,7 @@ gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
 	const guchar *buf;
 	gint blen;
 
-	drivername = gnome_print_config_get (job->config, "Settings.Engine.Backend.Driver");
+	drivername = (gchar *) gnome_print_config_get (job->config, (const guchar *) "Settings.Engine.Backend.Driver");
 
 	if (drivername == NULL)
 		return FALSE;
@@ -462,7 +528,7 @@ gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
 
 	*retval = GNOME_PRINT_ERROR_UNKNOWN;
 	
-	gnome_print_config_get_boolean (job->config, "Settings.Output.Job.PrintToFile", &print_to_file);
+	gnome_print_config_get_boolean (job->config, (const guchar *) "Settings.Output.Job.PrintToFile", &print_to_file);
 	if (!print_to_file) {
 		g_warning ("Metadata printer should always be print to file");
 		goto metadata_printer_done;
@@ -479,7 +545,7 @@ gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
 
 	gnome_print_transport_open  (transport);
 	gnome_print_transport_write (transport, buf, blen);
-	gnome_print_transport_write (transport, "GNOME_METAFILE_END", strlen ("GNOME_METAFILE_END"));
+	gnome_print_transport_write (transport, (const guchar *) "GNOME_METAFILE_END", strlen ("GNOME_METAFILE_END"));
 	gnome_print_transport_close (transport);
 	
 	*retval = GNOME_PRINT_OK;
@@ -504,8 +570,9 @@ gint
 gnome_print_job_print (GnomePrintJob *job)
 {
 	GnomePrintContext *ctx;
+	GnomePrintFilter *filter = NULL;
 	gint lpages, copies, nstacks, npages, nsheets;
-	gboolean collate, is_multipage, hwcopies;
+	gboolean collate, hwcopies;
 	gint stack;
 	const guchar *buf;
 	gint blen;
@@ -530,39 +597,25 @@ gnome_print_job_print (GnomePrintJob *job)
 	lpages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
 	if (lpages < 1)
 		return GNOME_PRINT_OK;
-	npages = lpages;
 
 	/* Special case where printer is a Metadata print to file */
 	if (gnome_print_job_metadata_printer (job, &ret))
 		return ret;
 
-	/* Create the "real" context */
+	/* Create the "real" context and set it up. */
 	ctx = gnome_print_context_new (job->config);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-
-	/* Update the layout data */
-	is_multipage = FALSE;
-	job_update_layout_data (job);
-	if (job->priv->LY_LIST) {
-		GnomePrintContext *mp;
-		/* Find out physical page count */
-		npages = (npages + job->priv->num_affines - 1) / job->priv->num_affines;
-		/* Create multipage context */
-		mp = gnome_print_multipage_new (ctx, job->priv->LY_LIST);
-		g_object_unref (G_OBJECT (ctx));
-		ctx = mp;
-		is_multipage = TRUE;
-	}
+	gnome_print_job_setup_context (job, ctx);
+	g_object_get (G_OBJECT (ctx), "filter", &filter, NULL);
 
 	collate = FALSE;
-	gnome_print_config_get_boolean (job->config, GNOME_PRINT_KEY_COLLATE, &collate);
+	gnome_print_config_get_boolean (job->config, (const guchar *) GNOME_PRINT_KEY_COLLATE, &collate);
 	copies = 1;
-	gnome_print_config_get_int (job->config, GNOME_PRINT_KEY_NUM_COPIES, &copies);
+	gnome_print_config_get_int (job->config, (const guchar *) GNOME_PRINT_KEY_NUM_COPIES, &copies);
 	hwcopies = FALSE;
 	gnome_print_config_get_boolean (job->config, 
 					collate 
-					? GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW
-					: GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW, 
+					? (const guchar *) GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW
+					: (const guchar *) GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW, 
 					&hwcopies);
 
 	if (hwcopies)
@@ -575,10 +628,8 @@ gnome_print_job_print (GnomePrintJob *job)
 		nstacks = 1;
 		nsheets = copies;
 	}
-	
-	buf  = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
-	blen = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
 
+	npages = (lpages + job->priv->num_affines - 1) / job->priv->num_affines;
 	for (stack = 0; stack < nstacks; stack++) {
 		gint page;
 		for (page = 0; page < npages; page++) {
@@ -588,13 +639,13 @@ gnome_print_job_print (GnomePrintJob *job)
 				/* Render physical page */
 				start = page * job->priv->num_affines;
 				for (i = start; (i < (start + job->priv->num_affines)) && (i < lpages); i++) {
-					ret = gnome_print_meta_render_data_page (ctx, buf, blen, i, TRUE);
+					ret = gnome_print_meta_render_page (GNOME_PRINT_META (job->meta),
+							ctx, i, TRUE);
 					g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
 				}
 				/* Flush physical page */
-				if (is_multipage) {
-					gnome_print_multipage_finish_page (GNOME_PRINT_MULTIPAGE (ctx));
-				}
+				if (filter)
+					gnome_print_filter_flush (filter);
 			}
 		}
 		if (stack + 1 < nstacks) {
@@ -623,34 +674,10 @@ gnome_print_job_print (GnomePrintJob *job)
 gint
 gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 {
-	const guchar *data;
-	gint len;
-	gint ret;
-
-	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
-	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	len = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (len > 0, GNOME_PRINT_ERROR_UNKNOWN);
-
-	job_update_layout_data (job);
-	if (job->priv->LY_LIST) {
-		GnomePrintContext *mp;
-		mp = gnome_print_multipage_new (ctx, job->priv->LY_LIST);
-		ret = gnome_print_meta_render_data (mp, data, len);
-		gnome_print_multipage_finish_page (GNOME_PRINT_MULTIPAGE (mp));
-		g_object_unref (G_OBJECT (mp));
-	} else {
-		ret = gnome_print_meta_render_data (ctx, data, len);
-	}
-
-	return ret;
+	gnome_print_job_setup_context (job, ctx);
+	return gnome_print_meta_render (GNOME_PRINT_META (job->meta), ctx);
 }
 
 /**
@@ -667,51 +694,15 @@ gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 gint
 gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint page, gboolean pageops)
 {
-	const guchar *data;
-	gint len, pages;
-	gint ret;
+	GnomePrintContext *pc;
 
-	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
-	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	len = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (len > 0, GNOME_PRINT_ERROR_UNKNOWN);
-	pages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
-
-	job_update_layout_data (job);
-	if (job->priv->LY_LIST) {
-		GnomePrintContext *meta, *mp;
-		gint start, p;
-
-		meta = gnome_print_meta_new ();
-		g_return_val_if_fail (meta != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-		/* Create wrapper multipage */
-		mp = gnome_print_multipage_new (meta, job->priv->LY_LIST);
-		start = page * job->priv->num_affines;
-		/* Render job pages */
-		for (p = start; (p < pages) && (p < start + job->priv->num_affines); p++) {
-			gnome_print_meta_render_data_page (mp, data, len, p, TRUE);
-		}
-		/* Finish multipage */
-		gnome_print_context_close (mp);
-		g_object_unref (G_OBJECT (mp));
-		/* Render page */
-		data = gnome_print_meta_get_buffer (GNOME_PRINT_META (meta));
-		len = gnome_print_meta_get_length (GNOME_PRINT_META (meta));
-		ret = gnome_print_meta_render_data_page (ctx, data, len, 0, pageops);
-		/* Release meta */
-		g_object_unref (G_OBJECT (meta));
-	} else {
-		ret = gnome_print_meta_render_data_page (ctx, data, len, page, pageops);
-	}
-
-	return ret;
+	pc = g_object_new (GNOME_TYPE_PRINT_META, NULL);
+	gnome_print_job_setup_context (job, pc);
+	gnome_print_meta_render (GNOME_PRINT_META (job->meta), pc);
+	gnome_print_meta_render_page (GNOME_PRINT_META (pc), ctx, page, pageops);
+	g_object_unref (G_OBJECT (pc));
 }
 
 /* We need:
@@ -748,7 +739,7 @@ gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint pa
 #endif
 
 static void
-job_update_layout_data (GnomePrintJob *job)
+gnome_print_job_update_layout_data (GnomePrintJob *job)
 {
 	ArtDRect area, r;
 	gdouble t;
@@ -866,28 +857,28 @@ job_parse_config_data (GnomePrintJob *job)
 
 
 	/* Physical size */
-	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_WIDTH, &job->priv->pw, &unit)) {
+	if (gnome_print_config_get_length (job->config, (const guchar *) GNOME_PRINT_KEY_PAPER_WIDTH, &job->priv->pw, &unit)) {
 		gnome_print_convert_distance (&job->priv->pw, unit, GNOME_PRINT_PS_UNIT);
 	}
-	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &job->priv->ph, &unit)) {
+	if (gnome_print_config_get_length (job->config, (const guchar *) GNOME_PRINT_KEY_PAPER_HEIGHT, &job->priv->ph, &unit)) {
 		gnome_print_convert_distance (&job->priv->ph, unit, GNOME_PRINT_PS_UNIT);
 	}
 	/* Physical orientation */
-	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, job->priv->porient);
+	gnome_print_config_get_transform (job->config, (const guchar *) GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, job->priv->porient);
 	/* Logical orientation */
-	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, job->priv->lorient);
+	gnome_print_config_get_transform (job->config, (const guchar *) GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, job->priv->lorient);
 	/* Layout size */
-	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_WIDTH, &job->priv->lyw);
-	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_HEIGHT, &job->priv->lyh);
+	gnome_print_config_get_double (job->config, (const guchar *) GNOME_PRINT_KEY_LAYOUT_WIDTH, &job->priv->lyw);
+	gnome_print_config_get_double (job->config, (const guchar *) GNOME_PRINT_KEY_LAYOUT_HEIGHT, &job->priv->lyh);
 
 	/* Now come the affines */
-	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (job->config), GNOME_PRINT_KEY_LAYOUT);
+	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (job->config), (const guchar *) GNOME_PRINT_KEY_LAYOUT);
 	if (layout) {
 		gint numlp;
 		numlp = 0;
-		if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
+		if (gpa_node_get_int_path_value (layout, (const guchar *) "LogicalPages", &numlp) && (numlp > 0)) {
 			GPANode *pages;
-			pages = gpa_node_get_child_from_path (layout, "Pages");
+			pages = gpa_node_get_child_from_path (layout, (const guchar *) "Pages");
 			if (pages) {
 				GPANode *page;
 				gdouble *affines;
@@ -898,7 +889,7 @@ job_parse_config_data (GnomePrintJob *job)
 				while (pagenum < numlp) {
 					guchar *transform;
 					guchar *child_id;
-					child_id = g_strdup_printf ("LP%d", pagenum);
+					child_id = (guchar *) g_strdup_printf ("LP%d", pagenum);
 					page = gpa_node_get_child_from_path (pages, child_id);
 					transform = gpa_node_get_value (page);
 					gpa_node_unref (page);
@@ -1101,7 +1092,7 @@ gnome_print_parse_transform (guchar *str, gdouble *transform)
 	art_affine_identity (transform);
 
 	p = str;
-	p = strchr (str, '(');
+	p = (guchar *) strchr ((gchar *) str, '(');
 	if (!p)
 		return FALSE;
 	p += 1;
@@ -1109,24 +1100,24 @@ gnome_print_parse_transform (guchar *str, gdouble *transform)
 		return FALSE;
 	for (i = 0; i < 6; i++) {
 		while (*p && g_ascii_isspace (*p)) p += 1;
-		if (!strncmp (p, "SQRT2", 5)) {
+		if (!strncmp ((gchar *) p, "SQRT2", 5)) {
 			t[i] = M_SQRT2;
-			e = p + 5;
-		} else if (!strncmp (p, "-SQRT2", 6)) {
+			e = (gchar *) p + 5;
+		} else if (!strncmp ((gchar *) p, "-SQRT2", 6)) {
 			t[i] = -M_SQRT2;
-			e = p + 6;
-		} else if (!strncmp (p, "SQRT1_2", 7)) {
+			e = (gchar *) p + 6;
+		} else if (!strncmp ((gchar *) p, "SQRT1_2", 7)) {
 			t[i] = M_SQRT1_2;
-			e = p + 7;
-		} else if (!strncmp (p, "-SQRT1_2", 8)) {
+			e = (gchar *) p + 7;
+		} else if (!strncmp ((gchar *) p, "-SQRT1_2", 8)) {
 			t[i] = -M_SQRT1_2;
-			e = p + 8;
+			e = (gchar *) p + 8;
 		} else {
-			t[i] = g_ascii_strtod (p, &e);
+			t[i] = g_ascii_strtod ((gchar *) p, &e);
 		}
 		if (e == (gchar *) p)
 			return FALSE;
-		p = e;
+		p = (guchar *) e;
 	}
 
 	memcpy (transform, t, 6 * sizeof (gdouble));
@@ -1148,13 +1139,13 @@ gint
 gnome_print_job_print_to_file (GnomePrintJob *job, gchar *output)
 {
 	if (output) {
-		gnome_print_config_set (job->config, "Settings.Transport.Backend",    "file");
-		gnome_print_config_set (job->config, GNOME_PRINT_KEY_OUTPUT_FILENAME, output);
+		gnome_print_config_set (job->config, (const guchar *) "Settings.Transport.Backend",    (const guchar *) "file");
+		gnome_print_config_set (job->config, (const guchar *) GNOME_PRINT_KEY_OUTPUT_FILENAME, (const guchar *) output);
 	} else {
 		/* In the future we might want to use the default or even better,
 		 * go back to the prev. selected printer (Chema)
 		 */
-		gnome_print_config_set (job->config, "Settings.Transport.Backend", "lpr");
+		gnome_print_config_set (job->config, (const guchar *) "Settings.Transport.Backend", (const guchar *) "lpr");
 	}
 	
 	return GNOME_PRINT_OK;
