@@ -41,6 +41,7 @@ G_BEGIN_DECLS
 #include <libgnomeprint/gp-gc.h>
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-transport.h>
 
 #define GNOME_PRINT_CONTEXT_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k),   GNOME_TYPE_PRINT_CONTEXT, GnomePrintContextClass))
 #define GNOME_IS_PRINT_CONTEXT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k),   GNOME_TYPE_PRINT_CONTEXT))
@@ -48,18 +49,20 @@ G_BEGIN_DECLS
 
 #define NEEDED_SUBSETTING(font) ((font)->face->entry->type == GP_FONT_ENTRY_TRUETYPE)
 
-typedef struct _GnomePrintContextClass GnomePrintContextClass;
-typedef struct _GnomePrintTransport      GnomePrintTransport;
+typedef struct _GnomePrintContextPrivate GnomePrintContextPrivate;
+typedef struct _GnomePrintContextClass   GnomePrintContextClass;
 
 struct _GnomePrintContext {
-	   GObject object;
+	GObject object;
 
-	   GnomePrintConfig *config;
-	   GnomePrintTransport *transport;
+	GnomePrintConfig *config;
+	GnomePrintTransport *transport;
 
-	   GPGC * gc;
-	   gboolean haspage;
-	   gint pages;
+	GPGC * gc;
+	gboolean haspage;
+	gint pages;
+
+	GnomePrintContextPrivate *priv;
 };
 
 struct _GnomePrintContextClass {
@@ -86,6 +89,7 @@ struct _GnomePrintContextClass {
 
 gint gnome_print_context_construct (GnomePrintContext *ctx, GnomePrintConfig *config);
 gint gnome_print_context_create_transport (GnomePrintContext *ctx);
+GnomePrintContext *gnome_print_context_new_from_module_name (const gchar *);
 
 gint gnome_print_clip_bpath_rule     (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule);
 gint gnome_print_fill_bpath_rule     (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule);
@@ -93,6 +97,20 @@ gint gnome_print_stroke_bpath        (GnomePrintContext *pc, const ArtBpath *bpa
 gint gnome_print_image_transform     (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint w, gint h, gint rowstride, gint ch);
 gint gnome_print_glyphlist_transform (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList *gl);
 
+/* Pass through */
+gint gnome_print_beginpage_real           (GnomePrintContext *, const guchar *);
+gint gnome_print_showpage_real            (GnomePrintContext *);
+gint gnome_print_gsave_real               (GnomePrintContext *);
+gint gnome_print_grestore_real            (GnomePrintContext *);
+gint gnome_print_clip_bpath_rule_real     (GnomePrintContext *, const ArtBpath *, ArtWindRule);
+gint gnome_print_fill_bpath_rule_real     (GnomePrintContext *, const ArtBpath *, ArtWindRule);
+gint gnome_print_stroke_bpath_real        (GnomePrintContext *, const ArtBpath *);
+gint gnome_print_image_transform_real     (GnomePrintContext *, const gdouble *, const guchar *, gint, gint, gint, gint);
+gint gnome_print_glyphlist_transform_real (GnomePrintContext *, const gdouble *, GnomeGlyphList *);
+gint gnome_print_setrgbcolor_real         (GnomePrintContext *, gdouble, gdouble, gdouble);
+gint gnome_print_setopacity_real          (GnomePrintContext *, gdouble);
+gint gnome_print_setlinewidth_real        (GnomePrintContext *, double);
+
 typedef struct _GnomePrintLayout         GnomePrintLayout;
 typedef struct _GnomePrintLayoutPage     GnomePrintLayoutPage;
 typedef struct _GnomePrintLayoutData     GnomePrintLayoutData;
