@@ -274,13 +274,13 @@ gnome_print_ps2_construct (GnomePrintContext *ctx)
 	ps2->bbox.x1 = 210 * 72.0 / 25.4;
 	ps2->bbox.y1 = 297 * 72.0 / 25.4;
 
-	gnome_print_config_get_length (ctx->config, GNOME_PRINT_KEY_PAPER_WIDTH,  &ps2->bbox.x1, NULL);
-	gnome_print_config_get_length (ctx->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &ps2->bbox.y1, NULL);
+	gnome_print_config_get_length (ctx->config, (const guchar *) GNOME_PRINT_KEY_PAPER_WIDTH,  &ps2->bbox.x1, NULL);
+	gnome_print_config_get_length (ctx->config, (const guchar *) GNOME_PRINT_KEY_PAPER_HEIGHT, &ps2->bbox.y1, NULL);
 
 	if (ctx->config) {
 		gdouble pp2pa[6];
 		art_affine_identity (pp2pa);
-		if (gnome_print_config_get_transform (ctx->config, GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, pp2pa)) {
+		if (gnome_print_config_get_transform (ctx->config, (const guchar *) GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, pp2pa)) {
 			art_drect_affine_transform (&ps2->bbox, &ps2->bbox, pp2pa);
 			ps2->bbox.x1 -= ps2->bbox.x0;
 			ps2->bbox.y1 -= ps2->bbox.y0;
@@ -399,7 +399,7 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 	hex = g_new (gchar, gnome_print_encode_hex_wcs (w * ch));
 
 	for (r = 0; r < h; r++) {
-		hex_size = gnome_print_encode_hex (px + r * rowstride, hex, w * ch);
+		hex_size = gnome_print_encode_hex (px + r * rowstride, (guchar *) hex, w * ch);
 		problem |= fwrite (hex, sizeof (gchar), hex_size, ps2->buf);
 		problem |= gnome_print_ps2_puts (ps2, EOL);
 	}
@@ -613,16 +613,16 @@ gnome_print_ps2_setpagedevice_policies (GnomePrintContext *pc)
 	gint y = (ps2->bbox.y1 - ps2->bbox.y0);
 
 	gnome_print_config_get_int (pc->config,
-				    GNOME_PRINT_KEY_NUM_COPIES,
-				    &numcopies);
+				  (const guchar *) GNOME_PRINT_KEY_NUM_COPIES,
+				  &numcopies);
 	gnome_print_config_get_boolean (pc->config,
-					GNOME_PRINT_KEY_COLLATE,
+					(const guchar *) GNOME_PRINT_KEY_COLLATE,
 					&collate);
 	gnome_print_config_get_boolean (pc->config,
-					GNOME_PRINT_KEY_DUPLEX,
+					(const guchar *) GNOME_PRINT_KEY_DUPLEX,
 					&duplex);
 	gnome_print_config_get_boolean (pc->config,
-					GNOME_PRINT_KEY_TUMBLE,
+					(const guchar *) GNOME_PRINT_KEY_TUMBLE,
 					&tumble);
 	if (collate)
 		numcopies = 1;
@@ -653,16 +653,16 @@ gnome_print_ps2_requirements (GnomePrintContext *pc)
 	int numcopies = 1;
 
 	gnome_print_config_get_int (pc->config,
-				    GNOME_PRINT_KEY_NUM_COPIES,
+				    (const guchar *) GNOME_PRINT_KEY_NUM_COPIES,
 				    &numcopies);
 	gnome_print_config_get_boolean (pc->config,
-					GNOME_PRINT_KEY_COLLATE,
+					(const guchar *) GNOME_PRINT_KEY_COLLATE,
 					&collate);
 	gnome_print_config_get_boolean (pc->config,
-					GNOME_PRINT_KEY_DUPLEX,
+					(const guchar *) GNOME_PRINT_KEY_DUPLEX,
 					&duplex);
 	gnome_print_config_get_boolean (pc->config,
-					GNOME_PRINT_KEY_TUMBLE,
+					(const guchar *) GNOME_PRINT_KEY_TUMBLE,
 					&tumble);
 	if (collate)
 		numcopies = 1;
@@ -694,10 +694,10 @@ gnome_print_ps2_setpagedevice (GnomePrintContext *pc, gboolean allow_duplex)
 
 	if (allow_duplex) {
 		gnome_print_config_get_boolean (pc->config,
-						GNOME_PRINT_KEY_DUPLEX,
+						(const guchar *) GNOME_PRINT_KEY_DUPLEX,
 						&duplex);
 		gnome_print_config_get_boolean (pc->config,
-						GNOME_PRINT_KEY_TUMBLE,
+						(const guchar *) GNOME_PRINT_KEY_TUMBLE,
 						&tumble);
 	}
 
@@ -725,12 +725,13 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 	GnomePrintPs2Page *page;
 	gint number;
 	gboolean problem;
+	ArtBpath bpath[6];
 
 	number = ps2->pages ? ps2->pages->number : 0;
 
 	page = g_new (GnomePrintPs2Page, 1);
 	page->next = ps2->pages;
-	page->name = g_strdup (name);
+	page->name = g_strdup ((const gchar *) name);
 	page->number = number + 1;
 	page->shown = FALSE;
 	page->usedfonts = NULL;
@@ -747,7 +748,7 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 	if (ps2->new_copy) {
 		gboolean duplex = FALSE;
 		gnome_print_config_get_boolean (pc->config,
-						GNOME_PRINT_KEY_DUPLEX,
+						(const guchar *) GNOME_PRINT_KEY_DUPLEX,
 						&duplex);
 
 		if (duplex) {
@@ -758,16 +759,14 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 		}
 		ps2->new_copy = FALSE;
 	}
-	problem |= gnome_print_newpath (pc) ||
-	    gnome_print_moveto (pc, 0.0, 0.0) ||
-	    gnome_print_lineto (pc, ps2->bbox.x1, 0.0) ||
-	    gnome_print_lineto (pc, ps2->bbox.x1, ps2->bbox.y1) ||
-	    gnome_print_lineto (pc, 0.0, ps2->bbox.y1) ||
-	    gnome_print_lineto (pc, 0.0, 0.0) ||
-	    gnome_print_closepath (pc) ||
-	    gnome_print_clip (pc) ||
-	    gnome_print_newpath (pc);
 
+	bpath[0].code = ART_MOVETO; bpath[0].x3 = 0.;           bpath[0].y3 = 0.;
+	bpath[1].code = ART_LINETO; bpath[1].x3 = ps2->bbox.x1; bpath[1].y3 = 0.;
+	bpath[2].code = ART_LINETO; bpath[2].x3 = ps2->bbox.x1; bpath[2].y3 = ps2->bbox.y1;
+	bpath[3].code = ART_LINETO; bpath[3].x3 = 0.;           bpath[3].y3 = ps2->bbox.y1;
+	bpath[4].code = ART_LINETO; bpath[4].x3 = 0.;           bpath[4].y3 = 0.;
+	bpath[5].code = ART_END;
+	problem |= gnome_print_clip_bpath_rule_real (pc, bpath, ART_WIND_RULE_NONZERO);
 	g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
 
 	return GNOME_PRINT_OK;
@@ -832,7 +831,7 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 
 	/* Orientation ([2] Page 43) */
 	gnome_print_transport_printf (pc->transport, "%%%%Orientation: ");
-	orientation = gnome_print_config_get (pc->config, GNOME_PRINT_KEY_ORIENTATION);
+	orientation = (gchar *) gnome_print_config_get (pc->config, (const guchar *) GNOME_PRINT_KEY_ORIENTATION);
 	if (!orientation || strcmp ("R0", orientation) == 0) {
 		gnome_print_transport_printf (pc->transport, "Portrait");
 	} else if (strcmp ("R90", orientation) == 0) {
@@ -856,7 +855,7 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	gnome_print_transport_printf (pc->transport, "%%%%Pages: %d" EOL, ps2->pages->number);
 	gnome_print_transport_printf (pc->transport, "%%%%PageOrder: Ascend" EOL);
 
-	name = gnome_print_config_get (pc->config,  GNOME_PRINT_KEY_DOCUMENT_NAME);
+	name = (gchar *) gnome_print_config_get (pc->config, (const guchar *) GNOME_PRINT_KEY_DOCUMENT_NAME);
 	if (name) {
 		gnome_print_transport_printf (pc->transport, "%%%%Title: %s" EOL, name);
 		g_free (name);
@@ -1047,15 +1046,15 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font,
 
 	if (subfont_flag) {
                 if (subfont_number == 0)
-                        encodedstring = g_strdup_printf ("GnomeUni-%s",gnome_font->face->psname);
+                        encodedstring = (guchar *) g_strdup_printf ("GnomeUni-%s",gnome_font->face->psname);
                 else
-                        encodedstring = g_strdup_printf ("GnomeUni-%s_%03d",gnome_font->face->psname, subfont_number);
+                        encodedstring = (guchar *) g_strdup_printf ("GnomeUni-%s_%03d",gnome_font->face->psname, subfont_number);
 
 		if (ps2->selected_font &&
 		    ps2->selected_font->pso &&
 		    ps2->selected_font->pso->encodedname) {
 
-			if (!strcmp (encodedstring, ps2->selected_font->pso->encodedname) && (gnome_font_get_size (gnome_font) == ps2->selected_font->currentsize)) {
+			if (!strcmp ((gchar *) encodedstring, (gchar *) ps2->selected_font->pso->encodedname) && (gnome_font_get_size (gnome_font) == ps2->selected_font->currentsize)) {
 				g_free (encodedstring);
 				return FALSE;
 			}
@@ -1079,7 +1078,7 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font,
 
 	for (font = ps2->fonts; font != NULL; font = font->next) {
 		if (subfont_flag) {
-			if  (!strcmp (font->pso->encodedname, encodedstring)) {
+			if  (!strcmp ((gchar *) font->pso->encodedname, (gchar *) encodedstring)) {
 				g_free (encodedstring);
 				break;
 			}
