@@ -660,86 +660,6 @@ gnome_print_strokepath (GnomePrintContext *pc)
 	return gnome_print_bpath (pc, gp_gc_get_currentpath (pc->gc), FALSE);
 }
 
-/**
- * gnome_print_setrgbcolor:
- * @pc: A #GnomePrintContext
- * @r: Red channel value
- * @g: Green channel value
- * @b: Blue channel value
- *
- * Sets color in graphic state to RGB triplet. This does not imply anything
- * about which colorspace is used internally.
- * Channel values are clamped to 0.0 - 1.0 region, 0.0 meaning minimum.
- *
- * Returns: #GNOME_PRINT_OK or positive value on success, negative error
- * code on failure.
- */
-int
-gnome_print_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
-{
-	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-
-	gp_gc_set_rgbcolor (pc->gc, r, g, b);
-
-	return GNOME_PRINT_OK;
-}
-
-/**
- * gnome_print_setopacity:
- * @pc: A #GnomePrintContext
- * @opacity: Opacity value
- *
- * Sets painting opacity in graphic state to given value.
- * Value is clamped to 0.0 - 1.0 region, 0.0 meaning full transparency and
- * 1.0 completely opaque paint.
- *
- * Returns: #GNOME_PRINT_OK or positive value on success, negative error
- * code on failure.
- */
-int
-gnome_print_setopacity (GnomePrintContext *pc, double opacity)
-{
-	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-
-	gp_gc_set_opacity (pc->gc, opacity);
-
-	return GNOME_PRINT_OK;
-}
-
-/**
- * gnome_print_setlinewidth:
- * @pc: A #GnomePrintContext
- * @width: Line width in user coordinates
- *
- * Sets line width in graphic state to given value.
- * Value is given in user coordinates, so effective line width depends on
- * CTM at the moment of #gnome_print_stroke or #gnome_print_strokepath.
- * Line width is always uniform in all directions, regardless of stretch
- * factor of CTM.
- * Default line width is 1.0 in user coordinates.
- *
- * Returns: #GNOME_PRINT_OK or positive value on success, negative error
- * code on failure.
- */
-int
-gnome_print_setlinewidth (GnomePrintContext *pc, double width)
-{
-	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-
-	gp_gc_set_linewidth (pc->gc, width);
-
-	return GNOME_PRINT_OK;
-}
-
 /**
  * gnome_print_setmiterlimit:
  * @pc: A #GnomePrintContext
@@ -949,7 +869,7 @@ gnome_print_show (GnomePrintContext *pc, const guchar *text)
 	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 	g_return_val_if_fail (text != NULL, GNOME_PRINT_ERROR_BADVALUE);
 
-	return gnome_print_show_sized (pc, text, strlen (text));
+	return gnome_print_show_sized (pc, text, strlen ((const gchar *) text));
 }
 
 /**
@@ -986,7 +906,7 @@ gnome_print_show_sized (GnomePrintContext *pc, const guchar *text, int bytes)
 	if (bytes < 1)
 		return GNOME_PRINT_OK;
 
-	g_return_val_if_fail (g_utf8_validate (text, bytes, &invalid), GNOME_PRINT_ERROR_TEXTCORRUPT);
+	g_return_val_if_fail (g_utf8_validate ((const gchar *) text, bytes, &invalid), GNOME_PRINT_ERROR_TEXTCORRUPT);
 
 	font = gp_gc_get_font (pc->gc);
 	g_return_val_if_fail (font != NULL, GNOME_PRINT_ERROR_UNKNOWN);
@@ -1083,6 +1003,21 @@ gnome_print_translate (GnomePrintContext *pc, double x, double y)
  * Convenience methods
  */
 
+/**
+ * gnome_print_line_stroked:
+ * @pc: A #GnomePrintContext
+ * @x0: X position of start point in user coordinates
+ * @y0: Y position of start point in user coordinates
+ * @x1: X position of end point in user coordinates
+ * @y1: Y position of end point in user coordinates
+ *
+ * A convenience function to draw a line.  Uses 
+ * #gnome_print_stroke to do the actual work.
+ * 
+ * Returns: #GNOME_PRINT_OK or positive value on success, negative error code
+ *          on failure.
+ *
+ **/
 gint
 gnome_print_line_stroked (GnomePrintContext *pc, gdouble x0, gdouble y0, gdouble x1, gdouble y1)
 {
@@ -1097,6 +1032,21 @@ gnome_print_line_stroked (GnomePrintContext *pc, gdouble x0, gdouble y0, gdouble
 	return gnome_print_stroke (pc);
 }
 
+/**
+ * gnome_print_rect_stroked:
+ * @pc: A #GnomePrintContext
+ * @x: X position of origin of rectangle in user coordinates
+ * @y: Y position of origin of rectangle in user coordinates
+ * @width: Width of the rectangle
+ * @height: Height of the rectange
+ *
+ * A convenience function to draw a rectangle. Uses 
+ * #gnome_print_stroke to do the actual work.
+ *
+ * Returns: #GNOME_PRINT_OK or positive value on success, negative error code
+ *          on failure. 
+ *
+ **/
 gint
 gnome_print_rect_stroked (GnomePrintContext *pc, gdouble x, gdouble y, gdouble width, gdouble height)
 {
@@ -1115,6 +1065,21 @@ gnome_print_rect_stroked (GnomePrintContext *pc, gdouble x, gdouble y, gdouble w
 	return gnome_print_stroke (pc);
 }
 
+/**
+ * gnome_print_rect_filled:
+ * @pc: A #GnomePrintContext
+ * @x: X position of origin of rectangle in user coordinates
+ * @y: Y position of origin of rectangle in user coordinates
+ * @width: Width of the rectangle
+ * @height: Height of the rectange
+ *
+ * A convenience function to draw a filled rectangle. Uses 
+ * #gnome_print_stroke to do the actual work.
+ *
+ * Returns: #GNOME_PRINT_OK or positive value on success, negative error code
+ *          on failure. 
+ *
+ **/
 gint
 gnome_print_rect_filled (GnomePrintContext *pc, gdouble x, gdouble y, gdouble width, gdouble height)
 {
