@@ -31,21 +31,32 @@
 #include <errno.h>
 #include <fcntl.h>
 
+#include <libgnomeprint/gnome-print-filter.h>
+
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gp-gc-private.h>
 #include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-print-ps2.h>
 #include <libgnomeprint/gnome-print-pdf.h>
-#include <libgnomeprint/gnome-print-frgba.h>
+#include <libgnomeprint/gnome-print-meta.h>
+#include <libgnomeprint/gnome-print-rbuf.h>
 
 #ifdef ENABLE_SVG
 #include <libgnomeprint/gnome-print-svg.h>
 #endif
 
+#ifdef HAVE_GDI
+#include <libgnomeprint/gnome-print-gdi.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
+gchar *gnome_print_locale_dir = GNOMELOCALEDIR;
+gchar *gnome_print_modules_dir = GNOME_PRINT_MODULES_DIR;
+gchar *gnome_print_data_dir = GNOME_PRINT_DATA_DIR;
+
 #ifdef HAVE_MMAP
 /* For the buffer stuff, remove when the buffer stuff is moved out here */
 #include <sys/mman.h>
@@ -71,13 +82,30 @@
 #define O_BINARY 0
 #endif
 
+enum {
+	PROP_0,
+	PROP_CONFIG,
+	PROP_TRANSPORT,
+	PROP_FILTER
+};
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
-static void gnome_print_context_init (GnomePrintContext *pc);
-static void gnome_print_context_finalize (GObject *object);
 
 static GObjectClass *parent_class = NULL;
 
+#define C(result) { gint re = (result); if (re < 0) { return re; }}
+
+struct _GnomePrintContextPrivate {
+	GnomePrintFilter *filter;
+};
+
+static void
+gnome_print_context_init (GnomePrintContext *pc)
+{
+	pc->gc = gp_gc_new ();
+	pc->priv = g_new0 (GnomePrintContextPrivate, 1);
+}
+
 GType
 gnome_print_context_get_type (void)
 {
@@ -98,34 +126,77 @@ gnome_print_context_get_type (void)
 }
 
 static void
-gnome_print_context_class_init (GnomePrintContextClass *klass)
+gnome_print_context_get_property (GObject *object, guint n, GValue *v,
+		GParamSpec *pspec)
 {
-	GObjectClass *object_class;
+	GnomePrintContext *context = GNOME_PRINT_CONTEXT (object);
 
-	object_class = (GObjectClass*) klass;
-
-	parent_class = g_type_class_peek_parent (klass);
-
-	object_class->finalize = gnome_print_context_finalize;
+	switch (n) {
+	case PROP_CONFIG:
+		g_value_set_object (v, context->config);
+		break;
+	case PROP_TRANSPORT:
+		g_value_set_object (v, context->transport);
+		break;
+	case PROP_FILTER:
+		g_value_set_object (v, context->priv->filter);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, n, pspec);
+	}
 }
 
 static void
-gnome_print_context_init (GnomePrintContext *pc)
+gnome_print_context_set_property (GObject *object, guint n, const GValue *v,
+		GParamSpec *pspec)
 {
-	pc->config = NULL;
-	pc->transport = NULL;
+	GnomePrintContext *context = GNOME_PRINT_CONTEXT (object);
+	GnomePrintFilter *filter;
 
-	pc->gc = gp_gc_new ();
-	pc->haspage = FALSE;
-	pc->pages = 0;
+	switch (n) {
+	case PROP_CONFIG:
+		if (context->config)
+			g_object_unref (context->config);
+		context->config = g_value_get_object (v);
+		if (context->config)
+			g_object_ref (context->config);
+		break;
+	case PROP_TRANSPORT:
+		if (context->transport)
+			g_object_unref (context->transport);
+		context->transport = g_value_get_object (v);
+		if (context->transport)
+			g_object_ref (context->transport);
+		break;
+	case PROP_FILTER:
+		if (context->priv->filter) {
+			g_object_unref (G_OBJECT (context->priv->filter));
+			context->priv->filter = NULL;
+		}
+		filter = g_value_get_object (v);
+		if (filter) {
+			g_object_ref (G_OBJECT (filter));
+			context->priv->filter = filter;
+		}
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, n, pspec);
+	}
 }
 
 static void
 gnome_print_context_finalize (GObject *object)
 {
-	GnomePrintContext *pc;
+	GnomePrintContext *pc = GNOME_PRINT_CONTEXT (object);
 
-	pc = GNOME_PRINT_CONTEXT (object);
+	if (pc->priv) {
+		if (pc->priv->filter) {
+			g_object_unref (G_OBJECT (pc->priv->filter));
+			pc->priv->filter = NULL;
+		}
+		g_free (pc->priv);
+		pc->priv = NULL;
+	}
 
 	if (pc->transport) {
 		g_warning ("file %s: line %d: Destroying Context with open transport", __FILE__, __LINE__);
@@ -134,14 +205,40 @@ gnome_print_context_finalize (GObject *object)
 	}
 
 	if (pc->config) {
-		pc->config = gnome_print_config_unref (pc->config);
+		gnome_print_config_unref (pc->config);
+		pc->config = NULL;
 	}
 
-	gp_gc_unref (pc->gc);
+	if (pc->gc) {
+		gp_gc_unref (pc->gc);
+		pc->gc = NULL;
+	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gnome_print_context_class_init (GnomePrintContextClass *klass)
+{
+	GObjectClass *object_class;
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class = (GObjectClass*) klass;
+	object_class->finalize = gnome_print_context_finalize;
+	object_class->get_property = gnome_print_context_get_property;
+	object_class->set_property = gnome_print_context_set_property;
+	g_object_class_install_property (object_class, PROP_CONFIG,
+			g_param_spec_object ("config", "Config", "Config",
+				GNOME_TYPE_PRINT_CONFIG, G_PARAM_READWRITE));
+	g_object_class_install_property (object_class, PROP_TRANSPORT,
+			g_param_spec_object ("transport", "Transport", "Transport",
+				GNOME_TYPE_PRINT_TRANSPORT, G_PARAM_READWRITE));
+	g_object_class_install_property (object_class, PROP_FILTER,
+			g_param_spec_object ("filter", _("Filter"), _("Filter"),
+				GNOME_TYPE_PRINT_FILTER, G_PARAM_READWRITE));
+}
+
 gint
 gnome_print_context_construct (GnomePrintContext *pc, GnomePrintConfig *config)
 {
@@ -179,6 +276,153 @@ gnome_print_context_create_transport (GnomePrintContext *pc)
 	return GNOME_PRINT_OK;
 }
 
+gint
+gnome_print_beginpage_real (GnomePrintContext *pc, const guchar *name)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (name != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (!pc->haspage, GNOME_PRINT_ERROR_BADVALUE);
+
+	pc->pages++;
+	gp_gc_reset (pc->gc);
+	pc->haspage = TRUE;
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage (pc, name));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_showpage_real (GnomePrintContext *pc)
+{
+	gint result = GNOME_PRINT_OK;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->showpage)
+		result = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->showpage (pc);
+	pc->haspage = FALSE;
+	C (result);
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_grestore_real (GnomePrintContext *pc)
+{
+	gint result = GNOME_PRINT_OK;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->grestore)
+		result = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->grestore (pc);
+	gp_gc_grestore (pc->gc);
+	C (result);
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_gsave_real (GnomePrintContext *pc)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	gp_gc_gsave (pc->gc);
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->gsave)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->gsave (pc));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_fill_bpath_rule_real (GnomePrintContext *pc,
+				  const ArtBpath *bpath, ArtWindRule rule)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->fill)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->fill (pc, bpath, rule)); 
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_clip_bpath_rule_real (GnomePrintContext *pc,
+				  const ArtBpath *bpath, ArtWindRule rule)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->clip)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->clip (pc, bpath, rule));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_stroke_bpath_real (GnomePrintContext *pc, const ArtBpath *bpath)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->stroke)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->stroke (pc, bpath));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_image_transform_real (GnomePrintContext *pc, const gdouble *a,
+		const guchar *px, gint w, gint h, gint r, gint ch)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->image)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->image (pc, a, px, w, h, r, ch));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_glyphlist_transform_real (GnomePrintContext *pc,
+				      const gdouble *a, GnomeGlyphList *l)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->glyphlist)
+		C (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->glyphlist (pc, a, l));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_setrgbcolor_real (GnomePrintContext *pc, gdouble r, gdouble g, gdouble b)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	C (gp_gc_set_rgbcolor (pc->gc, r, g, b));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_setopacity_real (GnomePrintContext *pc, gdouble opacity)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	C (gp_gc_set_opacity (pc->gc, opacity));
+
+	return GNOME_PRINT_OK;
+}
+
+gint
+gnome_print_setlinewidth_real (GnomePrintContext *pc, double width)
+{
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADVALUE);
+
+	C (gp_gc_set_linewidth (pc->gc, width));
+
+	return GNOME_PRINT_OK;
+}
+
 /* Direct class-method frontends */
 
 /**
@@ -200,31 +444,23 @@ gnome_print_context_create_transport (GnomePrintContext *pc)
 int
 gnome_print_beginpage (GnomePrintContext *pc, const guchar *name)
 {
-	guchar *real_name;
+	guchar *real_name = (guchar *) name;
+	gint ret;
 
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (!pc->haspage, GNOME_PRINT_ERROR_NOMATCH);
 
-	pc->pages++;
-	if (name == NULL) {
-		real_name = g_strdup_printf ("%d", pc->pages);
-	} else {
-		real_name = (guchar *) name;
-	}
-
-	gp_gc_reset (pc->gc);
-	pc->haspage = TRUE;
-
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage (pc, real_name);
-
-	if (name == NULL) {
+	if (!name)
+		real_name = (guchar *) g_strdup_printf ("%d", pc->pages + 1);
+	if (pc->priv->filter)
+		ret = gnome_print_filter_beginpage (pc->priv->filter, pc, real_name);
+	else
+		ret = gnome_print_beginpage_real (pc, real_name);
+	if (!name)
 		g_free (real_name);
-	}
 
-	return GNOME_PRINT_OK;
+	return ret;
 }
 
 /**
@@ -244,21 +480,15 @@ gnome_print_beginpage (GnomePrintContext *pc, const guchar *name)
 int
 gnome_print_showpage (GnomePrintContext *pc)
 {
-	gint ret;
-
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 
-	ret = GNOME_PRINT_OK;
-
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->showpage)
-		ret = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->showpage (pc);
-
-	pc->haspage = FALSE;
-
-	return ret;
+	if (pc->priv->filter)
+		return gnome_print_filter_showpage (pc->priv->filter);
+	else
+		return gnome_print_showpage_real (pc);
 }
 
 /**
@@ -311,21 +541,14 @@ gnome_print_end_doc (GnomePrintContext *pc)
 int
 gnome_print_gsave (GnomePrintContext *pc)
 {
-	gint ret;
-
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 
-	ret = GNOME_PRINT_OK;
-
-	gp_gc_gsave (pc->gc);
-
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->gsave)
-		ret = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->gsave (pc);
-
-	return ret;
+	if (pc->priv->filter)
+		return gnome_print_filter_gsave (pc->priv->filter);
+	else
+		return gnome_print_gsave_real (pc);
 }
 
 /**
@@ -341,21 +564,14 @@ gnome_print_gsave (GnomePrintContext *pc)
 int
 gnome_print_grestore (GnomePrintContext *pc)
 {
-	gint ret;
-
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 
-	ret = GNOME_PRINT_OK;
-
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->grestore)
-		ret = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->grestore (pc);
-
-	gp_gc_grestore (pc->gc);
-
-	return ret;
+	if (pc->priv->filter)
+		return gnome_print_filter_grestore (pc->priv->filter);
+	else
+		return gnome_print_grestore_real (pc);
 }
 
 int
@@ -367,10 +583,10 @@ gnome_print_clip_bpath_rule (GnomePrintContext *pc, const ArtBpath *bpath, ArtWi
 	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 	g_return_val_if_fail ((rule == ART_WIND_RULE_NONZERO) || (rule == ART_WIND_RULE_ODDEVEN), GNOME_PRINT_ERROR_BADVALUE);
 
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->clip)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->clip (pc, bpath, rule);
-
-	return GNOME_PRINT_OK;
+	if (pc->priv->filter)
+		return gnome_print_filter_clip (pc->priv->filter, bpath, rule);
+	else
+		return gnome_print_clip_bpath_rule_real (pc, bpath, rule);
 }
 
 int
@@ -379,13 +595,12 @@ gnome_print_fill_bpath_rule (GnomePrintContext *pc, const ArtBpath *bpath, ArtWi
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 	g_return_val_if_fail ((rule == ART_WIND_RULE_NONZERO) || (rule == ART_WIND_RULE_ODDEVEN), GNOME_PRINT_ERROR_BADVALUE);
 
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->fill)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->fill (pc, bpath, rule);
-
-	return GNOME_PRINT_OK;
+	if (pc->priv->filter)
+		return gnome_print_filter_fill (pc->priv->filter, bpath, rule);
+	else
+		return gnome_print_fill_bpath_rule_real (pc, bpath, rule);
 }
 
 int
@@ -394,13 +609,12 @@ gnome_print_stroke_bpath (GnomePrintContext *pc, const ArtBpath *bpath)
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 	g_return_val_if_fail (bpath != NULL, GNOME_PRINT_ERROR_BADVALUE);
 
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->stroke)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->stroke (pc, bpath);
-
-	return GNOME_PRINT_OK;
+	if (pc->priv->filter)
+		return gnome_print_filter_stroke (pc->priv->filter, bpath);
+	else
+		return gnome_print_stroke_bpath_real (pc, bpath);
 }
 
 int
@@ -409,7 +623,6 @@ gnome_print_image_transform (GnomePrintContext *pc, const gdouble *affine, const
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 	g_return_val_if_fail (affine != NULL, GNOME_PRINT_ERROR_BADVALUE);
 	g_return_val_if_fail (px != NULL, GNOME_PRINT_ERROR_BADVALUE);
 	g_return_val_if_fail (w > 0, GNOME_PRINT_ERROR_BADVALUE);
@@ -417,10 +630,10 @@ gnome_print_image_transform (GnomePrintContext *pc, const gdouble *affine, const
 	g_return_val_if_fail (rowstride >= ch * w, GNOME_PRINT_ERROR_BADVALUE);
 	g_return_val_if_fail ((ch == 1) || (ch == 3) || (ch == 4), GNOME_PRINT_ERROR_BADVALUE);
 
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->image)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->image (pc, affine, px, w, h, rowstride, ch);
-
-	return GNOME_PRINT_OK;
+	if (pc->priv->filter)
+		return gnome_print_filter_image (pc->priv->filter, affine, px, w, h, rowstride, ch);
+	else
+		return gnome_print_image_transform_real (pc, affine, px, w, h, rowstride, ch);
 }
 
 int
@@ -429,15 +642,94 @@ gnome_print_glyphlist_transform (GnomePrintContext *pc, const gdouble *affine, G
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
 	g_return_val_if_fail (affine != NULL, GNOME_PRINT_ERROR_BADVALUE);
 	g_return_val_if_fail (gl != NULL, GNOME_PRINT_ERROR_BADVALUE);
 	g_return_val_if_fail (GNOME_IS_GLYPHLIST (gl), GNOME_PRINT_ERROR_BADVALUE);
 
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->glyphlist)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->glyphlist (pc, affine, gl);
+	if (pc->priv->filter)
+		return gnome_print_filter_glyphlist (pc->priv->filter, affine, gl);
+	else
+		return gnome_print_glyphlist_transform_real (pc, affine, gl);
+}
 
-	return GNOME_PRINT_OK;
+/**
+ * gnome_print_setrgbcolor:
+ * @pc: A #GnomePrintContext
+ * @r: Red channel value
+ * @g: Green channel value
+ * @b: Blue channel value
+ *
+ * Sets color in graphic state to RGB triplet. This does not imply anything
+ * about which colorspace is used internally.
+ * Channel values are clamped to 0.0 - 1.0 region, 0.0 meaning minimum.
+ *
+ * Returns: #GNOME_PRINT_OK or positive value on success, negative error
+ * code on failure.
+ */
+int
+gnome_print_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
+{
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
+
+	if (pc->priv->filter)
+		return gnome_print_filter_setrgbcolor (pc->priv->filter, r, g, b);
+	else
+		return gnome_print_setrgbcolor_real (pc, r, g, b);
+}
+
+/**
+ * gnome_print_setopacity:
+ * @pc: A #GnomePrintContext
+ * @opacity: Opacity value
+ *
+ * Sets painting opacity in graphic state to given value.
+ * Value is clamped to 0.0 - 1.0 region, 0.0 meaning full transparency and
+ * 1.0 completely opaque paint.
+ *
+ * Returns: #GNOME_PRINT_OK or positive value on success, negative error
+ * code on failure.
+ */
+int
+gnome_print_setopacity (GnomePrintContext *pc, double opacity)
+{
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
+
+	if (pc->priv->filter)
+		return gnome_print_filter_setopacity (pc->priv->filter, opacity);
+	else
+		return gnome_print_setopacity_real (pc, opacity);
+}
+
+/**
+ * gnome_print_setlinewidth:
+ * @pc: A #GnomePrintContext
+ * @width: Line width in user coordinates
+ *
+ * Sets line width in graphic state to given value.
+ * Value is given in user coordinates, so effective line width depends on
+ * CTM at the moment of #gnome_print_stroke or #gnome_print_strokepath.
+ * Line width is always uniform in all directions, regardless of stretch
+ * factor of CTM.
+ * Default line width is 1.0 in user coordinates.
+ *
+ * Returns: #GNOME_PRINT_OK or positive value on success, negative error
+ * code on failure.
+ */
+int
+gnome_print_setlinewidth (GnomePrintContext *pc, double width)
+{
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
+
+	if (pc->priv->filter)
+		return gnome_print_filter_setlinewidth (pc->priv->filter, width);
+	else
+		return gnome_print_setlinewidth_real (pc, width);
 }
 
 /**
@@ -479,6 +771,19 @@ gnome_print_context_close (GnomePrintContext *pc)
 	return ret;
 }
 
+GnomePrintContext *
+gnome_print_context_new_from_module_name (const gchar *module_name)
+{
+	if (!module_name) return NULL;
+
+	if (!strcmp (module_name, "rbuf"))
+		return g_object_new (GNOME_TYPE_PRINT_RBUF, NULL);
+	if (!strcmp (module_name, "meta"))
+		return g_object_new (GNOME_TYPE_PRINT_META, NULL);
+
+	return NULL;
+}
+
 /**
  * gnome_print_context_new:
  * @config: GnomePrintConfig object to query print settings from
@@ -498,26 +803,19 @@ GnomePrintContext *
 gnome_print_context_new (GnomePrintConfig *config)
 {
 	GnomePrintContext *pc = NULL;
-	guchar *drivername;
+	gchar *drivername;
 
 	g_return_val_if_fail (config != NULL, NULL);
 
-	drivername = gnome_print_config_get (config, "Settings.Engine.Backend.Driver");
+	drivername = (gchar *) gnome_print_config_get (config, 
+			(const guchar *) "Settings.Engine.Backend.Driver");
 	if (drivername == NULL) {
 		drivername = g_strdup ("gnome-print-ps");
 	}
 
 	if (strcmp (drivername, "gnome-print-ps") == 0) {
-		GnomePrintContext *ps;
-		ps = gnome_print_ps2_new (config);
-		if (ps == NULL)
-			return NULL;
-		pc = gnome_print_frgba_new (ps);
-		if (pc == NULL)
-			return NULL;
-		g_object_unref (G_OBJECT (ps));
 		g_free (drivername);
-		return pc;
+		return gnome_print_ps2_new (config);
 	}
 
 	if (strcmp (drivername, "gnome-print-pdf") == 0) {
@@ -538,6 +836,16 @@ gnome_print_context_new (GnomePrintConfig *config)
 	}
 #endif
 
+#ifdef HAVE_GDI
+	if (strcmp (drivername, "gnome-print-gdi") == 0) {
+		pc = gnome_print_gdi_new (config);
+		if (pc == NULL)
+			return NULL;
+		g_free (drivername);
+		return pc;
+	}
+#endif
+
 	if (pc == NULL)
 		g_warning ("Could not create context for driver: %s", drivername);
 
@@ -558,6 +866,7 @@ gnome_print_buffer_munmap (GnomePrintBuffer *b)
 #elif defined G_OS_WIN32
 		if (b->was_mmaped)
 			UnmapViewOfFile (b->buf);
+		else
 #endif
 			g_free (b->buf);
 	}
@@ -585,7 +894,7 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 	b->was_mmaped = FALSE;
 	b->fd = -1;
 
-	fh = open (file_name, O_RDONLY | O_BINARY);
+	fh = open ((const gchar *) file_name, O_RDONLY | O_BINARY);
 	if (fh < 0) {
 		g_warning ("Can't open \"%s\"", file_name);
 		return GNOME_PRINT_ERROR_UNKNOWN;
@@ -600,9 +909,21 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 	b->buf = mmap (NULL, s.st_size, PROT_READ, MAP_SHARED, fh, 0);
 #elif defined G_OS_WIN32
 	{
-		HANDLE handle = CreateFileMapping ((HANDLE)_get_osfhandle (fh), NULL, PAGE_READONLY, 0, 0, NULL);
+		HANDLE handle;
+
+		handle = CreateFileMapping ((HANDLE) _get_osfhandle (fh), NULL, PAGE_READONLY, 0,
+		s.st_size, NULL);
+		if (!handle) {
+			g_warning ("Can't CreateFileMapping file %s", file_name);
+			return GNOME_PRINT_ERROR_UNKNOWN;
+		}
 		b->buf = MapViewOfFile (handle, FILE_MAP_READ, 0, 0, 0);
 		CloseHandle (handle);
+		if (!b->buf) {
+			g_warning ("Can't MapViewOfFile file %s", file_name);
+			return GNOME_PRINT_ERROR_UNKNOWN;
+		}
+		b->buf_size = s.st_size;
 	}
 #endif
 
@@ -652,3 +973,37 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 
 	return GNOME_PRINT_OK;
 }
+
+#ifdef G_OS_WIN32
+BOOL APIENTRY DllMain (HANDLE hModule, 
+                       DWORD  ul_reason_for_call, 
+                       LPVOID lpReserved)
+{
+	char *gnome_print_prefix;
+
+	switch (ul_reason_for_call)
+	{
+		case DLL_PROCESS_ATTACH:
+			gnome_print_prefix = g_win32_get_package_installation_directory (NULL, 
+				"libgnomeprint-2-2-0.dll");
+			if (gnome_print_prefix) {
+				gnome_print_locale_dir = g_build_filename (
+					gnome_print_prefix, "share", "locale", NULL);
+				gnome_print_modules_dir = g_build_filename (
+					gnome_print_prefix, "lib", "libgnomeprint", VERSION, "modules", NULL);
+				gnome_print_data_dir = g_build_filename (
+					gnome_print_prefix, "share", "libgnomeprint", VERSION, NULL);
+			}
+
+			break;
+		case DLL_PROCESS_DETACH:
+			g_free (gnome_print_locale_dir);
+			g_free (gnome_print_modules_dir);
+			g_free (gnome_print_data_dir);
+
+			break;
+	}
+
+	return TRUE;
+}
+#endif
